package newsticker.rssutils;
import java.io.IOException;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class RSSItem {

	private String title;
	private URL link;
	private String description;
	private Date time;

	public RSSItem(URL link, String title, String description, Date time) {
		this.title = title;
		this.link = link;
		this.description = description;
		this.time = time;
	}

	public String getTitle() {
		return title;
	}

	public URL getLink() {
		return link;
	}

	public String getDescription() {
		return description;
	}

	public Date getTimestamp() {
		return time;
	}

	static RSSItem parseItem(String item) {
		RSSItem parsedItem = null;
		try {
			String title;
			URL link;
			String description;
			Date timestamp = null;

			String lowerCase = item.toLowerCase();

			// Case: find titles and retrieve text
			int beginTag = lowerCase.indexOf("<title>");
			int endTag = lowerCase.indexOf("</title>");
			title = item.substring(beginTag + 7, endTag);

			// Case: find links and retrieve text
			beginTag = lowerCase.indexOf("<link>");
			endTag = lowerCase.indexOf("</link>");
			link = new URL(item.substring(beginTag + 6, endTag));

			// Case: find descriptions and retrieve text
			beginTag = lowerCase.indexOf("<description>");
			endTag = lowerCase.indexOf("</description>");
			description = item.substring(beginTag + 13, endTag);

			// Case: find time stamps and retrieve text.
			// Only do so if the feed uses time stamps
			beginTag = lowerCase.indexOf("<pubdate>");
			endTag = lowerCase.indexOf("</pubdate>");
			if (beginTag >= 0 && endTag >= 0) {
				DateFormat dateFormatter = new SimpleDateFormat(
						"EEE, d MMM yyyy HH:mm:ss Z",Locale.US);
				timestamp = dateFormatter.parse(item.substring(beginTag + 9,
						endTag));
			}
			parsedItem = new RSSItem(link, title, description, timestamp);
		} catch (IOException ioe) {
			System.err.println("IOException: " + ioe.getMessage());
		} catch (ParseException pe) {
			System.err.println("Invalid date format in timestamp"
					+ pe.getMessage());
		}
		return parsedItem;

	}
}
